@@ -93,16 +93,17 @@ impl CharRanges {
 
     /// Tests whether the given input character matches this instruction.
     #[inline(always)] // About ~5-15% more throughput then `#[inline]`
-    pub fn matches(&self, c: Char) -> bool {
+    pub fn matches(&self, c: Char) -> Option<usize> {
         // This speeds up the `match_class_unicode` benchmark by checking
         // some common cases quickly without binary search. e.g., Matching
         // a Unicode class on predominantly ASCII text.
-        for r in self.ranges.iter().take(4) {
+        for i in 0..cmp::min(self.ranges.len(), 4) {
+            let r = self.ranges[i];
             if c < r.0 {
-                return false;
+                return None;
             }
             if c <= r.1 {
-                return true;
+                return Some(i);
             }
         }
         self.ranges.binary_search_by(|r| {
@@ -113,7 +114,7 @@ impl CharRanges {
             } else {
                 Ordering::Equal
             }
-        }).is_ok()
+        }).ok()
     }
 }
 
