@@ -12,7 +12,6 @@ use std::borrow::Cow;
 use std::collections::HashMap;
 use std::collections::hash_map::Iter;
 use std::fmt;
-use std::ops::Index;
 #[cfg(feature = "pattern")]
 use std::str::pattern::{Pattern, Searcher, SearchStep};
 use std::str::FromStr;
@@ -378,11 +377,6 @@ impl Regex {
     /// assert_eq!(caps.at(1), Some("Citizen Kane"));
     /// assert_eq!(caps.at(2), Some("1941"));
     /// assert_eq!(caps.at(0), Some("'Citizen Kane' (1941)"));
-    /// // You can also access the groups by index using the Index notation.
-    /// // Note that this will panic on an invalid index.
-    /// assert_eq!(&caps[1], "Citizen Kane");
-    /// assert_eq!(&caps[2], "1941");
-    /// assert_eq!(&caps[0], "'Citizen Kane' (1941)");
     /// # }
     /// ```
     ///
@@ -401,21 +395,15 @@ impl Regex {
     /// assert_eq!(caps.name("title"), Some("Citizen Kane"));
     /// assert_eq!(caps.name("year"), Some("1941"));
     /// assert_eq!(caps.at(0), Some("'Citizen Kane' (1941)"));
-    /// // You can also access the groups by name using the Index notation.
-    /// // Note that this will panic on an invalid group name.
-    /// assert_eq!(&caps["title"], "Citizen Kane");
-    /// assert_eq!(&caps["year"], "1941");
-    /// assert_eq!(&caps[0], "'Citizen Kane' (1941)");
-    ///
     /// # }
     /// ```
     ///
     /// Here we name the capture groups, which we can access with the `name`
-    /// method or the `Index` notation with a `&str`. Note that the named capture groups
-    /// are still accessible with `at` or the `Index` notation with a `usize`.
+    /// method. Note that the named capture groups are still accessible with
+    /// `at`.
     ///
     /// The `0`th capture group is always unnamed, so it must always be
-    /// accessed with `at(0)` or `[0]`.
+    /// accessed with `at(0)`.
     pub fn captures<'t>(&self, text: &'t str) -> Option<Captures<'t>> {
         let mut caps = self.alloc_captures();
         if exec(self, &mut caps, text, 0) {
@@ -644,16 +632,15 @@ impl Regex {
         }
     }
 
-    /// Returns an iterator over the capture names.
-    pub fn capture_names<'r>(&'r self) -> CaptureNames<'r> {
+    #[doc(hidden)]
+    pub fn names_iter<'a>(&'a self) -> NamesIter<'a> {
         match *self {
-            Regex::Native(ref n) => CaptureNames::Native(n.names.iter()),
-            Regex::Dynamic(ref d) => CaptureNames::Dynamic(d.cap_names.iter())
+            Regex::Native(ref n) => NamesIter::Native(n.names.iter()),
+            Regex::Dynamic(ref d) => NamesIter::Dynamic(d.cap_names.iter())
         }
     }
 
-    /// Returns the number of captures.
-    pub fn captures_len(&self) -> usize {
+    fn names_len(&self) -> usize {
         match *self {
             Regex::Native(ref n) => n.names.len(),
             Regex::Dynamic(ref d) => d.cap_names.len()
@@ -668,34 +655,20 @@ impl Regex {
     }
 }
 
-/// Yields the names of all possible captures.
-/// `None` indicates an unnamed capture; the first element
-/// (capture 0, the whole matched region) is always unnamed.
-///
-/// `'r` is the lifetime of the compiled expression.
-pub enum CaptureNames<'r> {
-    #[doc(hidden)]
-    Native(::std::slice::Iter<'r, Option<&'static str>>),
-    #[doc(hidden)]
-    Dynamic(::std::slice::Iter<'r, Option<String>>)
+pub enum NamesIter<'a> {
+    Native(::std::slice::Iter<'a, Option<&'static str>>),
+    Dynamic(::std::slice::Iter<'a, Option<String>>)
 }
 
-impl<'r> Iterator for CaptureNames<'r> {
-    type Item=Option<&'r str>;
+impl<'a> Iterator for NamesIter<'a> {
+    type Item=Option<String>;
 
-    fn next(&mut self) -> Option<Option<&'r str>> {
+    fn next(&mut self) -> Option<Option<String>> {
         match *self {
-            CaptureNames::Native(ref mut i) =>
-                i.next().map(|o| *o),
-            CaptureNames::Dynamic(ref mut i) =>
-                i.next().as_ref().map(|o| o.as_ref().map(|s| s.as_ref())),
-        }
-    }
-
-    fn size_hint(&self) -> (usize, Option<usize>) {
-        match *self {
-            CaptureNames::Native(ref i)  => i.size_hint(),
-            CaptureNames::Dynamic(ref i) => i.size_hint(),
+            NamesIter::Native(ref mut i) =>
+                i.next().map(|x| x.map(|s| s.to_owned())),
+            NamesIter::Dynamic(ref mut i) =>
+                i.next().map(|x| x.as_ref().map(|s| s.to_owned())),
         }
     }
 }
@@ -738,10 +711,11 @@ impl<'t> Replacer for &'t str {
     }
 
     fn no_expand<'a>(&'a mut self) -> Option<Cow<'a, str>> {
-        // if there is a $ there may be an expansion
-        match self.find('$') {
-            Some(_) => None,
-            None => Some((*self).into()),
+        let re = Regex::new(REPLACE_EXPAND).unwrap();
+        if !re.is_match(self) {
+            Some((*self).into())
+        } else {
+            None
         }
     }
 }
@@ -839,13 +813,13 @@ impl<'t> Captures<'t> {
         locs: Vec<Option<usize>>,
     ) -> Captures<'t> {
         let named =
-            if re.captures_len() == 0 {
+            if re.names_len() == 0 {
                 None
             } else {
                 let mut named = HashMap::new();
-                for (i, name) in re.capture_names().enumerate() {
+                for (i, name) in re.names_iter().enumerate() {
                     if let Some(name) = name {
-                        named.insert(name.to_owned(), i);
+                        named.insert(name, i);
                     }
                 }
                 Some(named)
@@ -952,40 +926,6 @@ impl<'t> Captures<'t> {
     pub fn is_empty(&self) -> bool { self.len() == 0 }
 }
 
-/// Get a group by index.
-///
-/// # Panics
-/// If there is no group at the given index.
-impl<'t> Index<usize> for Captures<'t> {
-
-    type Output = str;
-
-    fn index<'a>(&'a self, i: usize) -> &'a str {
-        match self.at(i) {
-            None => panic!("no group at index '{}'", i),
-            Some(s) => s,
-        }
-    }
-
-}
-
-/// Get a group by name.
-///
-/// # Panics
-/// If there is no group named by the given value.
-impl<'t> Index<&'t str> for Captures<'t> {
-
-    type Output = str;
-
-    fn index<'a>(&'a self, name: &str) -> &'a str {
-        match self.name(name) {
-            None => panic!("no group named '{}'", name),
-            Some(ref s) => s,
-        }
-    }
-
-}
-
 /// An iterator over capture groups for a particular match of a regular
 /// expression.
 ///
@@ -1206,61 +1146,3 @@ fn exec(re: &Regex, caps: &mut CaptureIdxs, text: &str, start: usize) -> bool {
         Regex::Dynamic(ref prog) => prog.exec(caps, text, start),
     }
 }
-
-#[cfg(test)]
-mod test {
-    use super::{NoExpand, Regex};
-
-    #[test]
-    fn test_simple_expand() {
-        let re = Regex::new(r"(\w) (\w)").unwrap();
-        assert_eq!(re.replace_all("a b", "$2 $1"), "b a");
-    }
-
-    #[test]
-    fn test_literal_dollar() {
-        let re = Regex::new(r"(\w+) (\w+)").unwrap();
-        assert_eq!(re.replace_all("a b", "$1"), "a");
-        assert_eq!(re.replace_all("a b", "$$1"), "$1");  // $$ should become a $
-        assert_eq!(re.replace_all("a b", "$2 $$c $1"), "b $c a");
-    }
-
-    #[test]
-    fn test_no_expand() {
-        let re = Regex::new(r"(\w+)").unwrap();
-        assert_eq!(re.replace_all("a", NoExpand("$$1")), "$$1");
-        assert_eq!(re.replace_all("a", NoExpand("$1")), "$1");
-    }
-
-    #[test]
-    fn test_capture_names() {
-        let re = Regex::new(r"(.)(?P<a>.)").unwrap();
-        assert_eq!(re.capture_names().size_hint(), (3, Some(3)));
-        assert_eq!(re.capture_names().collect::<Vec<_>>(), [None, None, Some("a")]);
-    }
-
-    #[test]
-    fn test_cap_index() {
-        let re = Regex::new(r"^(?P<name>.+)$").unwrap();
-        let cap = re.captures("abc").unwrap();
-        assert_eq!(&cap[0], "abc");
-        assert_eq!(&cap[1], "abc");
-        assert_eq!(&cap["name"], "abc");
-    }
-
-    #[test]
-    #[should_panic]
-    fn test_cap_index_panic_usize() {
-        let re = Regex::new(r"^(?P<name>.+)$").unwrap();
-        let cap = re.captures("abc").unwrap();
-        let _ = cap[2];
-    }
-
-    #[test]
-    #[should_panic]
-    fn test_cap_index_panic_name() {
-        let re = Regex::new(r"^(?P<name>.+)$").unwrap();
-        let cap = re.captures("abc").unwrap();
-        let _ = cap["bad name"];
-    }
-}
